import pygame
import pygame.gfxdraw
import pygame_gui
from pygame_gui.elements import UITextEntryLine


class DrawHelper:
    @staticmethod
    def x_centered(size, location):
        return location[0] - size[0] // 2, location[1]

    @staticmethod
    def y_centered(size, location):
        return location[0], location[1] - size[1] // 2

    @staticmethod
    def xy_centered(size, location):
        return location[0] - size[0] // 2, location[1] - size[1] // 2

    @staticmethod
    def input_box(screen_size, surface, location, size, color, padding=0, function=None, name=None, radius=10):
        # tired of copy pasting, function is [string (type of function), ui manager, callback for adding it]
        h_scale = screen_size[0] // 20
        v_scale = screen_size[1] // 20
        size = (size[0] * h_scale - padding, size[1] * v_scale - padding)
        location = (location[0] * h_scale, location[1] * v_scale)
        location = DrawHelper.xy_centered(size, location)
        if function is not None:
            if function[0] == 'button':
                function[2](
                    pygame_gui.elements.UIButton(
                        relative_rect=pygame.Rect(location, size),
                        text='', manager=function[1],
                        object_id='#menu_button_rounded')
                )
                DrawHelper.rounded_rectangle(
                    surface, location, size[0],
                    size[1], radius, color)
            if function[0] == 'text':
                text_entry = UITextEntryLine(
                    relative_rect=pygame.Rect(location, size),
                    manager=function[1],
                    object_id=name)
                text_entry.set_forbidden_characters([',', '|', '&', '\\', '#'])
                function[2](text_entry)
                DrawHelper.rounded_rectangle(
                    surface, location, size[0],
                    size[1], radius, (61, 120, 180))
        else:
            DrawHelper.rounded_rectangle(
                surface, location, size[0],
                size[1], radius, color)

    @staticmethod
    def big_text(text, screen_size, surface, location, color, font, width=None):
        text_surface = pygame.font.Font.render(font, text, True, color, None)
        if width is None:
            pass
        else:
            h_scale = screen_size[0] // 20
            max_width = width * h_scale - h_scale * 0.5
            text_surface = pygame.font.Font.render(font, text, True, color, None)
            if text_surface.get_width() >= max_width:
                new_ratio = text_surface.get_width() / max_width
                new_height = int(text_surface.get_height() / new_ratio)
                new_width = int(text_surface.get_width() / new_ratio)
                text_surface = pygame.transform.scale(text_surface, (new_width, new_height))
        position = (int(screen_size[0] // 20 * location[0] - text_surface.get_width() / 2),
                    int(screen_size[1] // 20 * location[1] - text_surface.get_height() / 2))
        surface.blit(text_surface, position)

    @staticmethod
    def left_text(text, screen_size, surface, location, color, font, padding):
        text = pygame.font.Font.render(font, text, True, color, None)
        position = (int(screen_size[0] // 20 * location[0] + padding),
                    int(screen_size[1] // 20 * location[1]) - text.get_height() / 2)
        surface.blit(text, position)

    @staticmethod
    def intify(float_tuple):
        return int(float_tuple[0]), int(float_tuple[1])

    @staticmethod
    def rounded_rectangle(surface, location, width, height, radius, color):
        circle_x = DrawHelper.intify((location[0] + radius, location[0] + width - radius - 1))
        circle_y = DrawHelper.intify((location[1] + radius, location[1] + height - radius - 1))

        circle_centers = [
            (circle_x[0], circle_y[0]),
            (circle_x[0], circle_y[1]),
            (circle_x[1], circle_y[0]),
            (circle_x[1], circle_y[1])
        ]

        for points in circle_centers:
            pygame.gfxdraw.filled_circle(surface, points[0], points[1], radius, color)

        pygame.gfxdraw.box(surface, pygame.Rect(
            DrawHelper.intify((location[0] + radius, location[1])),
            DrawHelper.intify((width - radius * 2, height))
        ), color)

        pygame.gfxdraw.box(surface, pygame.Rect(
            DrawHelper.intify((location[0], location[1] + radius)),
            DrawHelper.intify((width, height - radius * 2))
        ), color)

    @staticmethod
    def rectangle(surface, location, width, height, color):

        h_scale = surface.get_width() // 20
        v_scale = surface.get_height() // 20

        location = DrawHelper.intify((h_scale * location[0], v_scale * location[1]))
        size = DrawHelper.intify((h_scale * width, v_scale * height))

        pygame.gfxdraw.box(surface, pygame.Rect(
            DrawHelper.intify(location),
            DrawHelper.intify(size)
        ), color)



import sys

import requests

s = requests.Session()
post_url = 'http://127.0.0.1:5000/'
post_url += sys.argv[1].split(',')[0] + '/' + sys.argv[1].split(',')[1] + '/' + 'post'
s.post(post_url, data={'msg': "quit"})



# Configurable options:
file1 = open('config.txt', 'r')
Lines = file1.readlines()
count = 0
for line in Lines:
    setting = line[:-1].split('=')
    if setting[0] == 'circle_tiles':
        CIRCLE_TILES = (setting[1] == 'True' or setting[1] == 1)
    if setting[0] == 'resolution_selection':
        if setting[1] in ['A', 'B', 'C', 'D', 'E']:
            resolution_selection = setting[1]
        else:
            resolution_selection = 'C'
file1.close()
# resolution_selection = 'C'

# Constants

RES_SCALE_DICT = {
    'A': ((1920, 1080), 1),
    'B': ((1200, 800), 75 / 100),
    'C': ((1600, 900), 83 / 100),
    'D': ((960, 540), 50 / 100),
    'E': ((1280, 720), 69 / 100)
}
FRAME_RATE = 60
RESOLUTION, PROTOTYPE_SCALE = RES_SCALE_DICT[resolution_selection]
SPACING = 3
FULLSCREEN = False
SCALE = 5 / 16
HEX_SIZE = 600 * SCALE * PROTOTYPE_SCALE
ACTION_SIZE = 70, 98
CARD_SIZE = 70, 98
WINDOW_CENTER = (RESOLUTION[0] / 2, RESOLUTION[1] / 2)

ACTION_WIDTH = ACTION_SIZE[0] * 6 + 7 * SPACING
INFO_WIDTH = 400 * PROTOTYPE_SCALE
DIALOG_SIZE = ACTION_WIDTH, 300 * PROTOTYPE_SCALE - 6 * SPACING
INFO_HEIGHT = RESOLUTION[1] - ACTION_SIZE[1] - DIALOG_SIZE[1] - 4 * SPACING

PLAYER_HEIGHT = int(INFO_WIDTH / 4)
PLAYER_SECTION_WIDTH = int((INFO_WIDTH - 4 * SPACING) / 5)
PLAYER_QUARTER_HEIGHT = int((PLAYER_HEIGHT - 2 * SPACING) / 4)
PLAYER_SECTION_SIZE = int((INFO_WIDTH - 3 * SPACING) / 8)

BANK_SECTION_SIZE = int((INFO_WIDTH - SPACING) / 8)
BANK_HEIGHT = BANK_SECTION_SIZE * 2



import pygame
import pygame_gui as pg_g
from frontiersman.client.GuiConstants import CARD_SIZE, SPACING

CARDS = ['Brick', 'Ore', 'Sheep', 'Wheat', 'Wood']
TITLE_SPACING = 28


class CardPickerGui(pg_g.elements.UIWindow):
    def __init__(self, top_left, manager, callback):
        super().__init__(pygame.Rect(top_left, ((CARD_SIZE[0] + SPACING) * 5 + 10 * SPACING, CARD_SIZE[1] + 10 * SPACING + TITLE_SPACING)), manager)
        self.manager = manager
        self.callback = callback
        self.cards = []
        self.close_window_button.disable()

        for i, card in enumerate(CARDS):
            self.cards.append(pg_g.elements.UIButton(
                relative_rect=pygame.Rect((i * (CARD_SIZE[0] + SPACING), 0), CARD_SIZE),
                text="",
                manager=self.manager,
                container=self,
                object_id="#" + card
            ))

    def handle_ui_button_pressed(self, event):
        try:
            index = self.cards.index(event.ui_element)
            self.callback(CARDS[index])
        except ValueError:
            pass



import os
import sys

import pygame
import pygame.gfxdraw

sys.path.insert(0, '../../frontiersman')
from frontiersman.main.DrawHelper import DrawHelper


class ClientGraphics:
    @staticmethod
    def top_and_bottom_bar(screen_size, small_font):
        graphics = pygame.Surface(screen_size, flags=pygame.SRCALPHA)

        color = (255, 255, 255)

        # bottom bar
        location = (10, 19)
        size = (20, 2)
        DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=0)

        # top bar
        location = (10, 0.5)
        size = (20, 1)
        DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=0)

        color = (0, 0, 0)

        # help button
        location = (1.5, 0.5)
        DrawHelper.big_text('Help', screen_size, graphics, location, color, small_font)

        # exit button
        location = (0.5, 0.5)
        DrawHelper.big_text('Exit', screen_size, graphics, location, color, small_font)

        return graphics

    @staticmethod
    def hand_area(screen_size, small_font, smaller_font):
        graphics = pygame.Surface(screen_size, flags=pygame.SRCALPHA)

        location = (10, 18.5)
        size = (8, 3)
        position = (6, 17)
        color = (255, 255, 255)
        DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=10)

        return graphics

    @staticmethod
    def action_buttons(screen_size, small_font, smaller_font, action=True):
        # def big_text(text, screen_size, surface, location, color, font, width=None):
        graphics = pygame.Surface(screen_size, flags=pygame.SRCALPHA)

        # end turn
        location = (18, 19)
        size = (4, 2)
        position = (16, 18)
        color = (255, 255, 255)
        DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=0)

        color = (0, 0, 0)

        DrawHelper.big_text('End Turn', screen_size, graphics, location, color, small_font)

        # Actions
        location = (15, 19)
        size = (2, 2)
        color = (255, 255, 255)
        DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=0)

        color = (0, 0, 0)

        DrawHelper.big_text('Actions', screen_size, graphics, location, color, small_font)

        if action:
            # Trade
            location = (15, 17)
            size = (2, 2)
            color = (255, 255, 255)
            DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=0)

            color = (0, 0, 0)

            DrawHelper.big_text('Trade', screen_size, graphics, location, color, small_font)

            # Buy
            location = (15, 15)
            size = (2, 2)
            color = (255, 255, 255)
            DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=0)

            color = (0, 0, 0)

            DrawHelper.big_text('Buy', screen_size, graphics, location, color, small_font)

            # Play dev card
            location = (15, 13)
            size = (2, 2)
            color = (255, 255, 255)
            DrawHelper.input_box(screen_size, graphics, location, size, color, padding=0, radius=0)

            color = (0, 0, 0)

            DrawHelper.big_text('Dev Card', screen_size, graphics, location, color, small_font)

        return graphics

    @staticmethod
    def test_info_cards(size, small_font, smaller_font):
        graphics = pygame.Surface(size, flags=pygame.SRCALPHA)

        graphics.blit(ClientGraphics.top_and_bottom_bar(size, smaller_font), (0, 0))

        graphics.blit(ClientGraphics.bank_info_card(size, smaller_font), (0, 0))

        graphics.blit(ClientGraphics.hand_area(size, small_font, smaller_font), (0, 0))

        graphics.blit(ClientGraphics.action_buttons(size, small_font, smaller_font), (0, 0))

        for player_number in range(0, 2):
            graphics.blit(ClientGraphics.player_info_card(size, small_font, smaller_font,
                                                          player=player_number,
                                                          name='Player ' + str(player_number),
                                                          num_cards=5,
                                                          inventory=(4, 6, 2, 15, 12, 5)),
                          (0, 0))
        return graphics

    @staticmethod
    def bank_info_card(screen_size, small_font, cards=None):
        bank_surface = pygame.Surface(screen_size, flags=pygame.SRCALPHA)

        h_scale = screen_size[0] // 20
        v_scale = screen_size[1] // 20

        position = (15, 0)
        location = (17.5, 1)

        bank_image_size = (80, 80)
        try:
            base_path = os.path.dirname(__file__)
            bank_image_load = pygame.image.load(os.path.join(base_path, '../assets/bank/bank_outlined_80x80.png'))
            bank_image = pygame.transform.scale(bank_image_load, bank_image_size)
        except:
            print('Failed to load bank image')
            bank_image = pygame.Surface(screen_size, flags=pygame.SRCALPHA)
            pygame.draw.rect(bank_image, pygame.Rect(DrawHelper.intify((position[0] * h_scale, position[1] * v_scale)),
                                                     bank_image_size))

        image_size = (45, 45)
        inventory_icons = ClientGraphics.scale_bank(image_size)

        if cards is None:
            card_list = [19, 19, 19, 19, 19, 20]
        else:
            card_list = cards

        # bank area
        color = (255, 255, 255)
        # color = (100, 100, 100)
        size = (5, 2)
        # bank area art
        color = (0, 0, 0, 120)
        DrawHelper.input_box(screen_size, bank_surface, location, size, color, padding=0, radius=5)
        color = (255, 255, 255)
        DrawHelper.input_box(screen_size, bank_surface, location, size, color, padding=2, radius=5)

        # maths
        pixel_position = (position[0] * h_scale, position[1] * v_scale)
        player_info_card_size = (screen_size[0] // 5, screen_size[1] // 10)
        player_info_card_center = (
            pixel_position[0] + player_info_card_size[0] // 2,
            pixel_position[1] + player_info_card_size[1] // 2)

        bank_image_area_center = (int(pixel_position[0] + h_scale - bank_image_size[0]),
                                  int(pixel_position[1] + v_scale - bank_image_size[1] // 2))

        # draw bank symbol
        bank_surface.blit(bank_image, bank_image_area_center)

        # draw icons
        icon_top_padding = 5
        icon_side_padding = 20
        icon_width = image_size[0]
        icon_tray_max_width = player_info_card_size[0] - 2 * icon_side_padding
        icon_padding = (icon_tray_max_width - 6 * icon_width) / 5

        index = 0
        for image in inventory_icons:
            bank_surface.blit(
                image, (int(pixel_position[0] + icon_side_padding + index * (icon_padding + icon_width) + h_scale),
                        int(player_info_card_center[1] - player_info_card_size[
                            1] // 4 - icon_width // 2 + icon_top_padding)))
            index += 1

        color = (0, 0, 0)

        # draw numbers for icons
        index = 0
        for number in card_list:
            text_surface = pygame.font.Font.render(small_font, str(number), True, color, None)
            bank_surface.blit(
                text_surface,
                (int(pixel_position[0] + icon_side_padding + index * (
                        icon_padding + icon_width) + icon_width // 2 - text_surface.get_width() // 2 + h_scale),
                 int(player_info_card_center[1] + player_info_card_size[1] // 4 - text_surface.get_height() // 2)))
            index += 1

        return bank_surface

    @staticmethod
    def player_info_card(screen_size, font, small_font, name='Zander', num_cards=0, inventory=None, player=0,
                         team_color=(120, 0, 0)):

        if inventory is None:
            player_thing_list = (5, 11, 3, 15, 7, 4)
        else:
            player_thing_list = inventory

        image_surface = pygame.Surface(screen_size, pygame.SRCALPHA)

        h_scale = screen_size[0] // 20
        v_scale = screen_size[1] // 20

        position = (16, 14 - player * 4)
        location = (18, 16 - player * 4)
        pixel_position = (position[0] * h_scale, position[1] * v_scale)
        player_info_card_size = (screen_size[0] // 5, screen_size[1] // 5)
        player_info_card_center = (
            pixel_position[0] + player_info_card_size[0] // 2,
            pixel_position[1] + player_info_card_size[1] // 2)

        image_size = (35, 35)
        inventory_icons = ClientGraphics.scale_logos(image_size)

        # card placeholder art
        card_width = 20
        card_height = 38
        card_side_padding = 10
        card_padding = 10
        max_width = player_info_card_size[0] - card_side_padding * 2
        card_art_thickness = 2
        card_art = pygame.Surface((card_width, card_height))
        card_art_face = pygame.Surface((card_width - card_art_thickness * 2, card_height - card_art_thickness * 2))
        card_art_face.fill((255, 255, 255))
        card_art.blit(card_art_face, (card_art_thickness, card_art_thickness))

        # first player
        size = (4, 4)
        color = (0, 0, 0, 120)
        DrawHelper.input_box(screen_size, image_surface, location, size, color, padding=1)
        color = (255, 255, 255)
        DrawHelper.input_box(screen_size, image_surface, location, size, color, padding=5)
        # player name
        location = (location[0], location[1] - 1.5)
        color = (0, 0, 0)
        DrawHelper.big_text(name, screen_size, image_surface, location, color, font)
        # player hand

        # make padding smaller so cards fit in box
        if num_cards * card_width + (num_cards - 1) * card_padding > max_width:
            card_padding = (max_width - card_side_padding * 2 - card_width * num_cards) / (num_cards - 1)

        # origin to draw cards from
        start_x = pixel_position[0] + card_side_padding \
                  + (max_width - (num_cards * card_width) - (num_cards - 1) * card_padding) // 2
        start_y = player_info_card_center[1] - player_info_card_size[1] // 8 - card_height // 2

        # draw cards
        for index in range(0, num_cards):
            image_surface.blit(card_art, (int(start_x + index * (card_width + card_padding)), int(start_y)))

        # draw icons
        icon_side_padding = 20
        icon_width = 35
        icon_tray_max_width = player_info_card_size[0] - 2 * icon_side_padding
        icon_padding = (icon_tray_max_width - 6 * icon_width) / 5

        bit_masks = ClientGraphics.scale_masks(image_size)
        for mask in bit_masks:
            mask.set_colorkey((255, 255, 255))

        index = 0
        for image in inventory_icons:
            if index in (3, 4, 5):
                color_surface = pygame.Surface(image_size, flags=pygame.SRCALPHA)
                color_surface.fill(team_color)
                color_surface.blit(bit_masks[index - 3], (0, 0))
                color_surface.set_colorkey((0, 0, 0))
                image_surface.blit(
                    color_surface, (int(pixel_position[0] + icon_side_padding + index * (icon_padding + icon_width)),
                                    int(player_info_card_center[1] + player_info_card_size[1] // 8 - icon_width // 2)))
            image_surface.blit(
                image, (int(pixel_position[0] + icon_side_padding + index * (icon_padding + icon_width)),
                        int(player_info_card_center[1] + player_info_card_size[1] // 8 - icon_width // 2)))
            index += 1

        # draw numbers for icons
        index = 0
        for number in player_thing_list:
            text_surface = pygame.font.Font.render(small_font, str(number), True, color, None)
            image_surface.blit(
                text_surface,
                (int(pixel_position[0] + icon_side_padding + index * (
                        icon_padding + icon_width) + icon_width // 2 - text_surface.get_width() // 2),
                 int(player_info_card_center[1] + 3 * player_info_card_size[1] // 8 - text_surface.get_height() // 2)))
            index += 1

        return image_surface

    @staticmethod
    def scale_logos(logo_size):
        base_path = os.path.dirname(__file__)
        logo_images = [
            pygame.image.load(os.path.join(base_path, '../assets/player/star.png')),
            pygame.image.load(os.path.join(base_path, '../assets/player/road.png')),
            pygame.image.load(os.path.join(base_path, '../assets/player/army.png')),
            pygame.image.load(os.path.join(base_path, '../assets/player/num_roads.png')),
            pygame.image.load(os.path.join(base_path, '../assets/player/settlement.png')),
            pygame.image.load(os.path.join(base_path, '../assets/player/city.png'))
        ]

        scaled = []

        for image in logo_images:
            scaled.append(pygame.transform.scale(image, logo_size))

        return scaled

    @staticmethod
    def scale_masks(logo_size):
        base_path = os.path.dirname(__file__)
        logo_images = [
            None,
            None,
            None,
            pygame.image.load(os.path.join(base_path, '../assets/player/road_mask.png')),
            pygame.image.load(os.path.join(base_path, '../assets/player/settlement_mask.png')),
            pygame.image.load(os.path.join(base_path, '../assets/player/city_mask.png'))
        ]

        scaled = []

        for image in logo_images:
            if image is not None:
                scaled.append(pygame.transform.scale(image, logo_size))

        return scaled

    @staticmethod
    def scale_bank(card_size):

        base_path = os.path.dirname(__file__)

        '''
        card_images = {
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/bank/wood_circle.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/bank/brick_circle.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/bank/sheep_circle.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/bank/wheat_circle.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/bank/ore_circle.png')),
            'Dev': pygame.image.load(os.path.join(base_path, '../assets/bank/card_back_square.png')),
        }
        '''

        card_images = {
            'Wood': pygame.image.load(os.path.join(base_path, '../assets/bank/wood_square.png')),
            'Brick': pygame.image.load(os.path.join(base_path, '../assets/bank/brick_square.png')),
            'Sheep': pygame.image.load(os.path.join(base_path, '../assets/bank/sheep_square.png')),
            'Wheat': pygame.image.load(os.path.join(base_path, '../assets/bank/wheat_square.png')),
            'Ore': pygame.image.load(os.path.join(base_path, '../assets/bank/ore_square.png')),
            'Dev': pygame.image.load(os.path.join(base_path, '../assets/bank/card_back_square.png')),
        }

        scaled = []

        for key in card_images:
            picture = card_images.get(key)
            scaled.append(pygame.transform.scale(picture, card_size))

        return scaled



import pygame
import pygame_gui as pg_g

from frontiersman.client import ClientHelper
from frontiersman.client.GuiConstants import INFO_WIDTH, SPACING, PLAYER_SECTION_SIZE, PLAYER_HEIGHT, BANK_SECTION_SIZE, BANK_HEIGHT

INFO_LOGOS = ClientHelper.TextureScale.scale_logos((PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE))
BANK_IMAGES = ClientHelper.TextureScale.scale_bank((BANK_SECTION_SIZE, BANK_SECTION_SIZE))
BANK_LOGO = ClientHelper.TextureScale.scale_bank_logo((BANK_SECTION_SIZE * 2, BANK_SECTION_SIZE * 2))


class PlayerInfo(pg_g.elements.UIPanel):
    def __init__(self, relative_rect, starting_layer_height, manager, font_size, anchors=None):
        super().__init__(relative_rect, starting_layer_height, manager, anchors=anchors)
        self.manager = manager
        self.font = pygame.font.SysFont('Comic Sans MS', font_size)

        self.bank_element = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, 0, INFO_WIDTH, BANK_HEIGHT),
            container=self,
            starting_layer_height=1,
            manager=self.manager)

        self.player_elements = []

    def render_text(self, text, size, gold_bg=False):
        if gold_bg:
            small_surface = self.font.render(text, True, (0,0,0))
        else:
            small_surface = self.font.render(text, True, (187, 187, 187))
        big_surface = pygame.Surface(size, pygame.SRCALPHA)

        text_pos = (big_surface.get_width() - small_surface.get_width()) / 2, (
                big_surface.get_height() - small_surface.get_height()) / 2
        bg_size = [small_surface.get_width() + 4 * SPACING, small_surface.get_height() + 4 * SPACING]
        bg_size[0] = max(bg_size[0], bg_size[1])

        bg_pos = text_pos[0] - (bg_size[0] - small_surface.get_width()) / 2, text_pos[1] - (
                bg_size[1] - small_surface.get_height()) / 2
        bg_rect = pygame.Rect(bg_pos, bg_size)

        
        if gold_bg:
            pygame.draw.rect(big_surface, (218,165,32), bg_rect)
        else:
            pygame.draw.rect(big_surface, (33, 40, 45), bg_rect)
        pygame.draw.rect(big_surface, (92, 96, 98), bg_rect, width=1)
        big_surface.blit(small_surface, text_pos)
        return big_surface

    def add_player_element(self, index, player, turn):
        curr_player_info = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, (PLAYER_HEIGHT + SPACING) * index + BANK_HEIGHT + SPACING,
                                      INFO_WIDTH - 2 * SPACING, PLAYER_HEIGHT + 2 * SPACING),
            container=self,
            starting_layer_height=1,
            manager=self.manager,
            object_id="#" + player['color']
        )

        name = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(0, 0, 2 * PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['name'], (2 * PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE), turn),
            manager=self.manager,
        )

        vp_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(name.relative_rect.right, name.relative_rect.top, PLAYER_SECTION_SIZE,
                                      PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['star'],
            manager=self.manager,
        )
        vp_value = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(vp_logo.relative_rect.right, vp_logo.relative_rect.top, PLAYER_SECTION_SIZE,
                                      PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['vp'], (PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE)),
            manager=self.manager,
        )

        longest_road_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(vp_value.relative_rect.right, vp_value.relative_rect.top, PLAYER_SECTION_SIZE,
                                      PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['road'],
            manager=self.manager,
        )
        longest_road_value = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(longest_road_logo.relative_rect.right, longest_road_logo.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['lr'], (PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE)),
            manager=self.manager,
        )

        largest_army_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(longest_road_value.relative_rect.right, longest_road_value.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            image_surface=INFO_LOGOS['army'],
            container=curr_player_info,
            manager=self.manager,
        )
        largest_army_value = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(largest_army_logo.relative_rect.right, largest_army_logo.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['la'], (PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE)),
            manager=self.manager,
        )

        hand_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(name.relative_rect.left, name.relative_rect.bottom, PLAYER_SECTION_SIZE,
                                      PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['cards'],
            manager=self.manager,
        )
        hand_value = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(hand_logo.relative_rect.right, hand_logo.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['hand'], (PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE)),
            manager=self.manager,
        )

        roads_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(hand_value.relative_rect.right, hand_value.relative_rect.top, PLAYER_SECTION_SIZE,
                                      PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['num_roads'],
            manager=self.manager,
        )
        roads_value = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(roads_logo.relative_rect.right, roads_logo.relative_rect.top, PLAYER_SECTION_SIZE,
                                      PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['roads'], (PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE)),
            manager=self.manager,
        )

        settlements_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(roads_value.relative_rect.right, roads_value.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['settlement'],
            manager=self.manager,
        )
        settlements_value = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(settlements_logo.relative_rect.right, roads_value.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['settlements'], (PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE)),
            manager=self.manager,
        )

        cities_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(settlements_value.relative_rect.right, settlements_value.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=INFO_LOGOS['city'],
            manager=self.manager,
        )
        cities_value = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(cities_logo.relative_rect.right, cities_logo.relative_rect.top,
                                      PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE),
            container=curr_player_info,
            image_surface=self.render_text(player['city'], (PLAYER_SECTION_SIZE, PLAYER_SECTION_SIZE)),
            manager=self.manager,
        )

        self.player_elements.append(curr_player_info)

    def set_player_info(self, current_player, enemy_players, turn_color):
        for player_element in self.player_elements:
            player_element.kill()
        if current_player.color==turn_color:
            turn=True
        else:
            turn=False
        curr_player = {
            'color': current_player.color.capitalize(),
            'name': current_player.name,
            'vp': str(current_player.victoryPoints) + (
                '' if current_player.hiddenVictoryPoints == 0 else ' (' + str(
                    current_player.hiddenVictoryPoints + current_player.victoryPoints) + ')'),
            'lr': str(current_player.longestRoad),
            'la': str(current_player.largestArmy),
            'hand': str(current_player.resourceHand.totalResources),
            'roads': str(current_player.numRoads),
            'settlements': str(current_player.numSettlements),
            'city': str(current_player.numCities),
        }
        self.add_player_element(0, curr_player,turn)

        for i, enemy in enumerate(enemy_players):
            if enemy.color==turn_color:
                turn=True
            else:
                turn=False
            player = {
                'color': enemy.color.capitalize(),
                'name': enemy.name,
                'vp': str(enemy.visibleVictoryPoints),
                'lr': str(enemy.longestRoad),
                'la': str(enemy.largestArmy),
                'hand': str(enemy.handSize),
                'roads': str(enemy.numRoads),
                'settlements': str(enemy.numSettlements),
                'city': str(enemy.numCities),
            }
            self.add_player_element(i + 1, player, turn)

    def set_bank_info(self, bank):
        self.bank_element.kill()
        self.bank_element = pg_g.elements.UIPanel(
            relative_rect=pygame.Rect(0, 0, INFO_WIDTH, BANK_HEIGHT + 2 * SPACING),
            container=self,
            starting_layer_height=1,
            manager=self.manager)

        bank_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(0, 0, BANK_SECTION_SIZE * 2, BANK_SECTION_SIZE * 2),
            container=self.bank_element,
            manager=self.manager,
            image_surface=BANK_LOGO,
        )

        wood_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(bank_logo.relative_rect.right, bank_logo.relative_rect.top, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Wood"],
            manager=self.manager,
            container=self.bank_element)

        brick_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wood_logo.relative_rect.right, wood_logo.relative_rect.top, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Brick"],
            manager=self.manager,
            container=self.bank_element)

        sheep_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(brick_logo.relative_rect.right, brick_logo.relative_rect.top, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Sheep"],
            manager=self.manager,
            container=self.bank_element)

        wheat_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(sheep_logo.relative_rect.right, sheep_logo.relative_rect.top, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Wheat"],
            manager=self.manager,
            container=self.bank_element)

        ore_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wheat_logo.relative_rect.right, wheat_logo.relative_rect.top, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Ore"],
            manager=self.manager,
            container=self.bank_element)

        dev_logo = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(ore_logo.relative_rect.right, ore_logo.relative_rect.top, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=BANK_IMAGES["Dev"],
            manager=self.manager,
            container=self.bank_element)

        wood_text = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wood_logo.relative_rect.left, wood_logo.relative_rect.bottom, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=self.render_text(str(bank.lumber), (BANK_SECTION_SIZE, BANK_SECTION_SIZE)),
            manager=self.manager,
            container=self.bank_element,
        )

        brick_text = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(brick_logo.relative_rect.left, brick_logo.relative_rect.bottom, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=self.render_text(str(bank.brick), (BANK_SECTION_SIZE, BANK_SECTION_SIZE)),
            manager=self.manager,
            container=self.bank_element,
        )

        sheep_text = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(sheep_logo.relative_rect.left, sheep_logo.relative_rect.bottom, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=self.render_text(str(bank.wool), (BANK_SECTION_SIZE, BANK_SECTION_SIZE)),
            manager=self.manager,
            container=self.bank_element,
        )

        wheat_text = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(wheat_logo.relative_rect.left, wheat_logo.relative_rect.bottom, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=self.render_text(str(bank.grain), (BANK_SECTION_SIZE, BANK_SECTION_SIZE)),
            manager=self.manager,
            container=self.bank_element,
        )

        ore_text = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(ore_logo.relative_rect.left, ore_logo.relative_rect.bottom, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=self.render_text(str(bank.ore), (BANK_SECTION_SIZE, BANK_SECTION_SIZE)),
            manager=self.manager,
            container=self.bank_element,
        )

        dev_text = pg_g.elements.UIImage(
            relative_rect=pygame.Rect(dev_logo.relative_rect.left, dev_logo.relative_rect.bottom, BANK_SECTION_SIZE,
                                      BANK_SECTION_SIZE),
            image_surface=self.render_text(str(bank.totalDevelopment), (BANK_SECTION_SIZE, BANK_SECTION_SIZE)),
            manager=self.manager,
            container=self.bank_element,
        )



import pygame


class WinWindow:
    def __init__(self, size, font, who_won='Zander', won=False):
        self.graphics = pygame.Surface(size, flags=pygame.SRCALPHA)

        self.message = []

        if won:
            self.message.append('You won the game!')
            self.message.append('Congratulations!')
        else:
            self.message.append(who_won + ' won the game.')
            self.message.append('Better luck next time.')

    def get_graphics(self):
        return self.graphics
